@@ -3,13 +3,22 @@ import sys
 import tqdm
 import logging
 import datatable as dt
-
+import gc
+import psutil
+import tempfile
 from collections import defaultdict
+import os
 
 log = logging.getLogger("my_logger")
 
 sys.setrecursionlimit(10 ** 6)
 
+# from https://softwareengineering.stackexchange.com/a/419985
+def line_estimation(filename, first_size=1 << 24):
+    with open(filename, "rb") as file:
+        buf = file.read(first_size)
+        return len(buf) // buf.count(b"\n")
+
 
 def read_and_filter_alns(
     aln,
@@ -46,28 +55,100 @@ def read_and_filter_alns(
     """
     dt.options.progress.clear_on_success = True
     dt.options.nthreads = threads
-    aln = dt.fread(
-        aln,
-        sep="\t",
-        header=False,
-        nthreads=threads,
-        columns=col_names[0],
-    )
-    aln[dt.bool8] = dt.int32
-    nalns = aln.shape[0]
-    # logging.info(f"Read {nalns:,} alignments. Getting basic statistics.")
 
-    logging.info(f"Pre-filtering: {nalns:,} alignments found")
-    logging.info(
-        f"Filtering alignments with bitscore >= {bitscore} and evalue <= {evalue}"
-    )
-    aln = aln[(dt.f.eVal < evalue) & (dt.f.bitScore > bitscore), :]
+    # We do a raw estimation of the number of alignments in a file. This is
+    # because we don't know how many alignments are in the file and they can be in
+    # in the order of billions.
+
+    logging.info("Getting a raw estimate of the number of alignments...")
+    nalns = os.path.getsize(aln) // line_estimation(aln)
+    logging.info(f"Approximately {nalns:,} alignments found.")
+
+    max_rows = int((2 ** 31) - 1)
+
+    # if the number of alignmentsis larger than the number of rows
+    # that can be read by fread, we need to read it by chunks instead
+    alns = []
+    if nalns > max_rows:
+        gc.collect()
+        logging.info(
+            f"Pre-filtering: {nalns:,} alignments found. This is more than {max_rows:,} alignments."
+        )
+        logging.info(
+            f"::: This is not supported at the moment. Trying to read and filter using chunks instead."
+        )
+
+        k, m = divmod(nalns, max_rows)
+
+        for i in range(k + 1):
+            logging.info(f"::: Reading chunk #{i+1:,}.")
+            up = max_rows * (i + 1)
+            if up > nalns:
+                up = nalns
+                aln_chunk = dt.fread(
+                    aln,
+                    sep="\t",
+                    header=False,
+                    nthreads=threads,
+                    columns=col_names[0],
+                    skip_to_line=int(max_rows * i),
+                )
+            else:
+                up = int(up - max_rows * i)
+                aln_chunk = dt.fread(
+                    aln,
+                    sep="\t",
+                    header=False,
+                    nthreads=threads,
+                    columns=col_names[0],
+                    skip_to_line=int(max_rows * i),
+                    max_nrows=up,
+                )
+            logging.info(
+                f"::: Filtering alignments in chunk #{i+1:,} with bitscore >= {bitscore} and evalue <= {evalue}"
+            )
+            alns.append(
+                aln_chunk[(dt.f.eVal < evalue) & (dt.f.bitScore > bitscore), :].copy(
+                    deep=True
+                )
+            )
+            del aln_chunk
+            gc.collect(generation=2)
+
+        logging.info(f"::: Concatenating chunks.")
+        aln = alns[0]
+        alns.pop(0)
+
+        if len(alns) > 0:
+            for i in alns:
+                aln.rbind(i)
+        if aln.shape[0] > max_rows:
+            logging.error(
+                f"The resulting table has more than {max_rows,} alignments and it is not supported at the moment."
+            )
+            exit(1)
+    else:
+        logging.info(f"Pre-filtering: {nalns:,} alignments found")
+        # logging.info(f"Read {nalns:,} alignments. Getting basic statistics.")
+
+        logging.info(
+            f"Filtering alignments with bitscore >= {bitscore} and evalue <= {evalue}"
+        )
+        aln = dt.fread(
+            aln,
+            sep="\t",
+            header=False,
+            nthreads=threads,
+            columns=col_names[0],
+        )
+        aln = aln[(dt.f.eVal < evalue) & (dt.f.bitScore > bitscore), :]
+
     nalns = aln.shape[0]
     logging.info(f"Post-filtering: {nalns:,} alignments found")
 
+    logging.info(f"Removing reads mapping multiple times to the same subject")
     aln = aln[:1, :, dt.by(dt.f.queryId, dt.f.subjectId), dt.sort(-dt.f.bitScore)]
     nalns = aln.shape[0]
-    logging.info(f"Removing reads mapping multiple times to the same subject")
     logging.info(f"::: Kept best bitScore alignments: {nalns:,}")
     return aln
 
@@ -324,20 +405,35 @@ def get_coverage_stats(df, trim=True):
     queries = defaultdict(int)
     gen_data = defaultdict(dict)
     rl = defaultdict(list)
-    for a, q, b, c, d, e in tqdm.tqdm(
-        zip(
-            df["Chromosome"],
-            df["Query"],
-            df["Start"],
-            df["End"],
-            df["slen"],
-            df["qlen"],
-        ),
+    # for a, q, b, c, d, e in tqdm.tqdm(
+    #     zip(
+    #         df["Chromosome"],
+    #         df["Query"],
+    #         df["Start"],
+    #         df["End"],
+    #         df["slen"],
+    #         df["qlen"],
+    #     ),
+    #     total=df.shape[0],
+    #     leave=False,
+    #     ncols=100,
+    #     desc=f"Alignments processed",
+    # ):
+    for i in tqdm.tqdm(
+        np.arange(df.shape[0]),
         total=df.shape[0],
         leave=False,
         ncols=100,
         desc=f"Alignments processed",
     ):
+        i = int(i)
+        a = df[i, "subjectId"]
+        q = df[i, "queryId"]
+        b = df[i, "subjectStart"]
+        c = df[i, "subjectEnd"]
+        d = df[i, "slen"]
+        e = df[i, "qlen"]
+
         b = b - 1
 
         if a in gen_data:
@@ -351,7 +447,7 @@ def get_coverage_stats(df, trim=True):
             gen_data[a]["n_alns"] += 1
             rl[a] += [e / 3]
     logging.info(
-        f"References will be dinamycally trimmed at 5'/3'-ends (half of the avg. read length)"
+        f"References will be dynamically trimmed at 5'/3'-ends (half of the avg. read length)"
     )
     stats = [
         get_stats_coverage(
@@ -399,7 +495,6 @@ def aggregate_gene_abundances(mapping_file, gene_abundances, threads=1):
         Returns:
             dt.Frame: The aggregated gene abundances
     """
-
     mappings = dt.fread(
         mapping_file, sep="\t", columns=["reference", "group"]
     ).to_pandas()
